using System;
using Microsoft.SPOT;
using System.IO.Ports;

namespace GPSTest
{
    class GPS
    {
        public SerialPort serialPort;
        public readonly object locker = new object();
        public byte[] buffer = new byte[1024];  //buffer
        public int bufferLength = 0;
        public int BufferMax = 1024;

        // This raises the event
        private void RaiseDataReceived()
        {
            var handler = this.DataReceived;
            if (handler != null)
            {
                handler();
            }
        }

        // This is the event
        public event DataReceivedEventHandler DataReceived;



        public string ReadLine()
        {
            string line = string.Empty;

            lock (this.locker)
            {
                for (int i = 0; i < this.bufferLength; i++)
                {
                    //if (this.buffer[i] == '\r' || this.buffer[i] == '\n')
                    if (this.buffer[i] == '\n')
                    {
                        this.buffer[i] = 0;
                        this.buffer[i-1] = 0;

                        line += new string(System.Text.Encoding.UTF8.GetChars(this.buffer, 0, i));

                        if (line != null && line != string.Empty)
                            //Debug.Print("Line read: {" + line + "}");

                        this.bufferLength = this.bufferLength - i - 1;

                        Array.Copy(this.buffer, i + 1, this.buffer, 0, this.bufferLength);

                        break;
                    }
                }
            }

            return line;
        }


        #region GPS SERIAL METHODS
        public void CloseGPSSerial()
        {
            serialPort.Flush();
            serialPort.DiscardInBuffer();
            serialPort.DiscardOutBuffer();
            serialPort.Close();
            serialPort = null;
        }

        public void OpenGPSSerial()
        {
            if (serialPort != null)
                if (serialPort.IsOpen)
                    CloseGPSSerial();

            serialPort = null;
            serialPort = new SerialPort("COM4", 9600, Parity.None, 8, StopBits.One); //gps
            serialPort.ReadTimeout = -1;  //in ms
            serialPort.Handshake = Handshake.None;  //flowcontrol = none
            serialPort.DataReceived += new SerialDataReceivedEventHandler(m_Sport_GPS_DataReceived);
            serialPort.Open();

        }

        private void m_Sport_GPS_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {


            lock (this.locker)
            {
                int bytesReceived = this.serialPort.Read(this.buffer, this.bufferLength, this.serialPort.BytesToRead);

                if (bytesReceived > 0)
                {
                    this.bufferLength += bytesReceived;

                    if (this.bufferLength >= BufferMax)
                        throw new Exception("Buffer overflow.");

                    this.RaiseDataReceived();
                }
            }


        }

        #endregion



    }
}
